﻿Imports System
Imports System.Runtime.InteropServices
Imports System.Runtime.CompilerServices

'*******************************************************************************************
' File      : bp2595.vb
' Function  : API for CPS BP2595 bipolar power supply
' Usage     : Include this file in VB.NET projects that control BP2595 supplies
' Copyright : Copyright (C) 2021 CPS
'*******************************************************************************************

Public Module bp2595

    Public Const BP2595_MAX_DEVICES As Integer = 8  ' Maximum number of 2595s connected to a host (limited by device selector rotary switch).
    Public Const INFINITE_WAIT As Integer = -1      ' Infinite wait time for blocking functions.

    Public Enum BP2595_ERR As Integer ' ERROR CODES ----------------
        OK = 0                      ' No error
        OPCODE = 1                  ' Command not recognized.
        ARGVAL = 2                  ' Illegal argument value.
        MODE_LOCKOUT = 3            ' Remote control prohibited in local mode; Local control prohibited in remote mode.
        FAULT_LOCKOUT = 4           ' Command failed due to fault condition (e.g., WriteSetpoint rejected while Overtemp).
        API_CLOSED = 20             ' API is closed.
        API_OPEN = 21               ' API is already open (attempting to open while already open).
        DEVICE_NOT_FOUND = 30       ' Specified device is not detected (can't find device with specified device ID).
        CANT_OPEN_DEVICE = 31       ' Failed to open device.
        DEVICE_CLOSED = 32          ' Device is closed.
        DEVICE_OPEN = 33            ' Device is already open.
        DUPLICATE_ADDRESS = 34      ' Two modules have the same address.
        WAS_SHUTDOWN = 40           ' USB connection was shutdown.
        READ = 41                   ' USB read error.
        WRITE = 42                  ' USB write error.
        OPENHIDLIB = 43             ' Failed to open HIDLIB.
        REPLY_MISMATCH = 50         ' Reply doesn't match last issued command.
        REPLY_VALUE = 51            ' Illegal reply value.
        TIMEOUT = 60                ' Timed out waiting for message from device.
        MALLOC = 70                 ' Memory allocation failed.
        CREATE_RESOURCE = 71        ' Failed to create mutex/semaphore/thread.
        RESOURCE_BUSY = 72          ' Resource is busy (locked by another thread).
        SYSTEM = 100                ' Unknown system error.
    End Enum

    Public Declare Function ErrString Lib "BP2595" Alias "BP2595_ErrString" (ByVal errcode As BP2595_ERR) As String

    <StructLayout(LayoutKind.Sequential)> _
    Public Structure VERSION                ' VERSION NUMBER ----------
        Public Major As Byte                    ' Major version.
        Public Minor As Byte                    ' Minor version.
        Public Build As UInt16                  ' Build number.
        Public Overrides Function ToString() As String
            Return Major.ToString & "." & Minor.ToString & "." & Build.ToString
        End Function
        Public Function ToUInt32() As UInt32
            Return (Convert.ToUInt32(Major) << 24) + (Convert.ToUInt32(Minor) << 16) + Convert.ToUInt32(Build)
        End Function
    End Structure


    '**************************************************************************************************************
    '**************************************************************************************************************
    '****************************  API OBJECT: SYSTEM OF ONE OR MORE 2595 DEVICES  ********************************
    '**************************************************************************************************************
    '**************************************************************************************************************

    Public Class Api

        Private Declare Function BP2595_OpenApi Lib "BP2595" (ByRef devflags As Integer) As BP2595_ERR
        Private Declare Function BP2595_CloseApi Lib "BP2595" () As BP2595_ERR
        Private Declare Function BP2595_GetApiVersion Lib "BP2595" (ByRef ver As VERSION) As BP2595_ERR

        Public Event ApiError(ByVal ErrMsg As String)

        ' Storage
        Private MyDevFlags As Integer
        Private MyVersion As VERSION

        ' API open/close -- returns: true if successful; false if failed
        Public Function Open() As Boolean
            Try
                Dim errcode = BP2595_OpenApi(MyDevFlags)   ' get DetectedDevice flags
                If (errcode = BP2595_ERR.OK) Then Return True
                RaiseEvent ApiError(ErrString(errcode))
            Catch ex As Exception
                RaiseEvent ApiError("Can't find BP2595.dll")
            End Try
            Return False
        End Function
        Public Sub Close()
            Try ' in case dll not found
                BP2595_CloseApi()
            Catch ex As Exception
                'ignore exception
            End Try
        End Sub

        ' API properties
        Public ReadOnly Property DevicesDetected() As Boolean
            Get
                Return (MyDevFlags <> 0)
            End Get
        End Property
        Public ReadOnly Property IsDeviceDetected(ByVal index As UInteger) As Boolean
            Get
                Return If(index < BP2595_MAX_DEVICES, (MyDevFlags And (1 << index)) <> 0, False)
            End Get
        End Property
        Public ReadOnly Property Version() As bp2595.VERSION
            Get
                Dim ver As bp2595.VERSION
                Dim errcode As Integer = BP2595_GetApiVersion(ver)
                If (errcode <> BP2595_ERR.OK) Then RaiseEvent ApiError(ErrString(errcode))
                Return ver
            End Get
        End Property

    End Class ' Api


    '**************************************************************************************************************
    '**************************************************************************************************************
    '********************************  DEVICE OBJECT: ONE BP2595 POWER SUPPLY UNIT  *******************************
    '**************************************************************************************************************
    '**************************************************************************************************************

    Public Class Device

        <Flags()> Private Enum STAT_FLAGS As UInt16  ' LIVE STATUS FLAGS -----------------
            UVLO = (1 << 0)         ' Undervoltage lockout: 1 = system input voltage too low; 0 = normal.
            OVERTEMP = (1 << 1)     ' Over-temperature: 1 = overtemp shutdown; 0 = normal.
            FAIL = (1 << 2)         ' Subsystem failed (relay, pos/neg supply): 1 = fail; 0 = normal.
            COM_TOUT = (1 << 3)     ' Communication timeout: 1 = timeout; 0 = normal.
            REMOTE = (1 << 4)       ' Control mode: 1 = remote via USB; 0 = local via front panel.
            HVEN = (1 << 5)         ' HV enable: 1 = HV enabled; 0 = HV disabled.
        End Enum

        <Flags()> Private Enum FAIL_FLAGS As Byte  ' SUBSYSTEM FAILURE STATUS FLAGS -------------
            RELAY_DRV = (1 << 0)    ' Relay failed to switch to target position.
            RELAY_FB = (1 << 1)     ' Relay feedback not detected.
            POS_SUPPLY = (1 << 2)   ' Positive supply malfunctioned.
            NEG_SUPPLY = (1 << 3)   ' Negative supply malfunctioned.
        End Enum

        <Flags()> Private Enum EVENT_LOG As Byte ' LATCHED ("STICKY") STATUS FLAGS -------------
            UVLO = (1 << 0)         ' Undervoltage lockout.
            OVERTEMP = (1 << 1)     ' Over-temperature.
            FAIL = (1 << 2)         ' Subsystem failed.
            COM_TOUT = (1 << 3)     ' Communication timeout.
            RESET = (1 << 4)        ' Device was reset.
            ATTRDEF = (1 << 5)      ' Valid personality stored in flash.
        End Enum

        <Flags()> Private Enum EV As UInt16    ' EVENT NOTIFICATION FLAGS -----------------
            UVLO = (1 << 0)         ' Undervoltage lockout change
            OVERTEMP = (1 << 1)     ' Overtemp condition change
            FAIL = (1 << 2)         ' Subsystem fail change.
            COM_TOUT = (1 << 3)     ' Communication timeout state change
            REMOTE = (1 << 4)       ' Local/remote mode change
            HVEN_CMD = (1 << 5)     ' HV enable changed by remote command
            HVEN_ASYNC = (1 << 6)   ' HV enable changed by local command
            SETPT_CMD = (1 << 7)    ' Setpoint changed by remote command
            SETPT_ASYNC = (1 << 8)  ' Setpoint changed by local command
            LOG_CHANGE = (1 << 14)  ' Status log changed ("sticky" status)
        End Enum

        <Flags()> Public Enum RST_FLAGS As UInt16
            RST_POR = 0             ' Power on
            RST_BOR = 1             ' Brown-out
            RST_WDTO = 4            ' Watchdog timeout
            RST_EXTR = 7            ' MCLR-invoked
            RST_IOP = 14            ' Illegal opcode or uninitialized W access
            RST_TRAP = 15           ' Trap conflict
        End Enum

        <StructLayout(LayoutKind.Sequential)> _
        Private Structure BP2595_VERSION_INFO        ' VERSION INFO ----------
            Public DevFw As bp2595.VERSION           ' Device firmware version number.
            Public PwbRev As Char                   ' Device hardware revision letter.
        End Structure

        <StructLayout(LayoutKind.Sequential)> _
        Public Structure DEV_METERS                 ' INTERNAL SUPPLY METERS ----------
            Public AbsOutV As UInt16                ' Absolute HV output voltage (unsigned volts).
            Public outUA As UInt16                  ' HV output current (uA).
            Public DeciDegC As UInt16               ' Temperature (0.1 C/bit).
        End Structure

        <StructLayout(LayoutKind.Sequential)> _
        Public Structure BP2595_METERS              ' API METER DATA ----------
            Public outV As Int32                    ' System output volts
            Public outUA As UInt16                  ' System output microamps
            Public DeciDegC As UInt16               ' Temperature (0.1 C/bit).
            Public inMV As UInt16                   ' System supply mV
            Public posMeters As DEV_METERS          ' Positive supply meters
            Public negMeters As DEV_METERS          ' Negative supply meters
        End Structure

        <StructLayout(LayoutKind.Sequential)> _
        Private Structure STATUS_SYS                ' STATUS/EVENT SYSTEM ----------
            Public EventFlags As EV                 ' Flags indicating event(s) that occurred since the most recent status read.
            Public LiveStatus As STAT_FLAGS         ' Live status flags.
            Public StatusLog As EVENT_LOG           ' Latched status events.
            Public FailFlags As FAIL_FLAGS          ' Subsystem fail flags.
        End Structure

        <StructLayout(LayoutKind.Sequential)> _
        Private Structure BP2595_STATUS             ' STATUS INFO ----------
            Public SetpointV As Int32               ' HV control setpoint in volts.
            Public StatusSys As STATUS_SYS          ' Status/event system.
        End Structure

        <StructLayout(LayoutKind.Sequential)> _
        Public Structure DEV_ATTR                   ' UNIPOLAR SUPPLY ATTRIBUTES ----------
            Public fs_vout_v As UInt16              ' Output voltage (absolute V) at full-scale native setpoint.
            Public fs_meter_out_v As UInt16         ' Output voltage (absolute V) at full-scale native Vout meter.
            Public fs_meter_out_ua As UInt16        ' Output current (absolute uA) at full-scale native Iout meter.
        End Structure

        <StructLayout(LayoutKind.Sequential)> _
        Public Structure BP2595_ATTR                ' SYSTEM PERSONALITY ATTRIBUTES ---------
            Public sernum As UInt32                 ' Serial number.
            Public output_max_v As UInt16           ' Output maximum V (absolute V). Setpoint amplitude is not allowed to exceed this (full-scale positive setpoint).
            Public output_min_v As UInt16           ' Output minimum V (absolute V). Setpoint amplitudes below this will turn off HV.
            Public rly_sw_thresh As UInt16          ' HV relay switching threshold (absolute V). Both 2600 output voltages must be below this to permit relay switching.
            Public devattr_pos As DEV_ATTR          ' Positive supply attributes.
            Public devattr_neg As DEV_ATTR          ' Negative supply attributes.
            Public rly_lockout_time As UInt16       ' Minimum time (milliseconds) between relay switching strobes; used to prevent rapid back-to-back strobes
            Public rly_strobe_time As UInt16        ' Duration of relay switching strobe (motor excitation) in milliseconds.
            Public fs_meter_vin_mv As UInt16        ' System input voltage at full-scale raw adc (determined by Vin-to-adc R divider ratio).
            Public uvlo_thresh As UInt16            ' Deactivate UVLO when Vin >= uvlo_thresh.
            Public uvlo_band As UInt16              ' Activate UVLO when Vin <= (uvlo_thresh - uvlo_band).
            Public safe_v_thresh As UInt16          ' System output voltage (absolute V) above this are "unsafe" and cause the HV warning LED to light.
            Public overtemp_thresh As UInt16        ' System temperatures (0.1 degC/bit units) > overtemp_thresh cause HV shutdown.
            Public overtemp_band As UInt16          ' System temperatures (0.1 degC/bit units) < (overtemp_thresh - overtemp_band) cancel HV shutdown.
            Public coarse_adj As Byte               ' Front panel coarse adjust step size (volts).
        End Structure

        <StructLayout(LayoutKind.Sequential)> _
        Private Structure TASKINFO                  ' FIRMWARE THREAD INFO
            <MarshalAs(UnmanagedType.ByValArray, SizeConst:=4)> Public Name As Char()
            Public StackMargin As UInt16
            Public ExecTime As UInt16
        End Structure

        ' Storage
        Private MyStatus As BP2595_STATUS               ' Device status
        Private MyMeters As BP2595_METERS               ' Meter data
        Private MyDevAddr As Integer                    ' Logical device address, matches address switch setting
        Private MyVersionInfo As BP2595_VERSION_INFO    ' Device firmware version and pwb rev
        Private MyGuiContext As Form                    ' Application's GUI context

        ' Async monitor threads.  
        Private StatusMonitorThread As Threading.Thread = New Threading.Thread(AddressOf StatusMonitorThreadFunction)
        Private MeterMonitorThread As Threading.Thread = New Threading.Thread(AddressOf MetersMonitorThreadFunction)

        ' API functions
        Private Declare Function BP2595_OpenDevice Lib "BP2595" (ByVal devaddr As Integer, ByRef status As BP2595_STATUS) As BP2595_ERR
        Private Declare Function BP2595_CloseDevice Lib "BP2595" (ByVal devaddr As Integer) As BP2595_ERR
        Private Declare Function BP2595_SetComWatchdog Lib "BP2595" (ByVal devaddr As Integer, ByVal interval As UInt16) As BP2595_ERR
        Private Declare Function BP2595_GetComWatchdog Lib "BP2595" (ByVal devaddr As Integer, ByRef interval As UInt16) As BP2595_ERR
        Private Declare Function BP2595_GetDeviceVersions Lib "BP2595" (ByVal devaddr As Integer, ByRef ver As BP2595_VERSION_INFO) As BP2595_ERR
        Private Declare Function BP2595_ProgramSetpoint Lib "BP2595" (ByVal devaddr As Integer, ByVal volts As Int32) As BP2595_ERR
        Private Declare Function BP2595_ProgramHvEnable Lib "BP2595" (ByVal devaddr As Integer, ByVal enable As Integer) As BP2595_ERR
        Private Declare Function BP2595_GetStatusUpdate Lib "BP2595" (ByVal devaddr As Integer, ByRef status As BP2595_STATUS, ByVal tmax As Integer) As BP2595_ERR
        Private Declare Function BP2595_ClearEventLog Lib "BP2595" (ByVal devaddr As Integer, ByVal flags As Byte) As BP2595_ERR
        Private Declare Function BP2595_SetMeterInterval Lib "BP2595" (ByVal devaddr As Integer, ByVal interval As UInt16) As BP2595_ERR
        Private Declare Function BP2595_GetMeterInterval Lib "BP2595" (ByVal devaddr As Integer, ByRef interval As UInt16) As BP2595_ERR
        Private Declare Function BP2595_GetMeterUpdate Lib "BP2595" (ByVal devaddr As Integer, ByRef meters As BP2595_METERS, ByVal tmax As Integer) As BP2595_ERR
        Private Declare Function BP2595_GetPersonality Lib "BP2595" (ByVal devaddr As Integer, ByRef attr As BP2595_ATTR) As BP2595_ERR

        '**********************************************************************************************
        '***************************************  ERRORS  *********************************************
        '**********************************************************************************************

        ' Error event
        Public Event DeviceError(ByVal ErrCode As Integer)

        ' Error detect
        Private Sub ErrorDetect(ByVal errcode As Integer)
            If (errcode <> 0) Then RaiseEvent DeviceError(errcode) ' Raise device error to app
        End Sub

        '**********************************************************************************************
        '**********************************  DEVICE OPEN/CLOSE  ***************************************
        '**********************************************************************************************

        ' Open device
        Public Sub Open(ByVal devaddr As UInteger, ByVal GuiContext As Form)
            If (devaddr >= bp2595.BP2595_MAX_DEVICES) Then
                RaiseEvent DeviceError(BP2595_ERR.ARGVAL)
            Else
                MyDevAddr = devaddr  ' Save logical device address (matches rotary switch settings on the target QD02)
                MyGuiContext = GuiContext                                       ' GUI context to which notifications will be sent.
                ErrorDetect(BP2595_OpenDevice(MyDevAddr, MyStatus))             ' Open device and read initial status
                ErrorDetect(BP2595_GetDeviceVersions(MyDevAddr, MyVersionInfo)) ' Read hardware version info
                StatusMonitorThread.Start()                                     ' Launch status monitor thread
                MeterMonitorThread.Start()                                      ' Launch meter monitor thread
                RaiseStatusChanges()                                            ' Send initial status to app
            End If
        End Sub

        ' Close device
        Public Sub Close()
            BP2595_CloseDevice(MyDevAddr)    ' Close device. This causes blocking api functions to return an error, which will terminate the status/meter monitor threads.
            StatusMonitorThread.Join()      ' Wait for status monitor thread to terminate.
            MeterMonitorThread.Join()       ' Wait for meters monitor thread to terminate.
        End Sub

        '**********************************************************************************************
        '***************************************  STATUS  *********************************************
        '**********************************************************************************************

        ' Status events
        Public Event FaultChange() ' UVLO or subsystem failed
        Public Event OvertempChange()
        Public Event ComTimeoutChange(ByVal Timeout As Boolean)
        Public Event RemoteControlChange(ByVal RemoteControl As Boolean)
        Public Event UsbSetpointChange()
        Public Event AsyncSetpointChange()
        Public Event UsbHvEnableChange(ByVal Enabled As Boolean)
        Public Event AsyncHvEnableChange(ByVal Enabled As Boolean)
        Public Event StatusLogChange()

        ' Status monitor thread
        Private Delegate Sub GUIInvokerStatusUpdate(ByVal status As BP2595_STATUS)
        Private Sub StatusMonitorThreadFunction()
            Dim status As New BP2595_STATUS  ' create status info buffer
            ' Repeat until error or device closed:
            While (BP2595_GetStatusUpdate(MyDevAddr, status, bp2595.INFINITE_WAIT) = BP2595_ERR.OK)        ' Wait for status update from device.
                MyGuiContext.BeginInvoke(New GUIInvokerStatusUpdate(AddressOf RaiseStatusUpdate), status)   ' Schedule call to RaiseStatusUpdate in GUI context.
            End While
        End Sub
        Private Sub RaiseStatusUpdate(ByVal status As BP2595_STATUS)
            MyStatus = status       ' Sync local status image to device
            RaiseStatusChanges()    ' Forward received status update to application
        End Sub

        ' Notify app that status has changed
        Private Sub RaiseStatusChanges()
            Dim v = MyStatus.StatusSys.EventFlags
            ' Report fault changes
            If (v And EV.UVLO) Then RaiseEvent FaultChange()
            If (v And EV.OVERTEMP) Then RaiseEvent OvertempChange()
            If (v And EV.FAIL) Then RaiseEvent FaultChange()
            If (v And EV.COM_TOUT) Then RaiseEvent ComTimeoutChange(Me.Status.ComTimeout)
            ' Report local-invoked control changes
            If (v And EV.REMOTE) Then RaiseEvent RemoteControlChange(Me.Status.RemoteControl)
            If (v And EV.HVEN_ASYNC) Then RaiseEvent AsyncHvEnableChange(Me.Status.HVEnabled)
            If (v And EV.SETPT_ASYNC) Then RaiseEvent AsyncSetpointChange()
            ' Report remote-invoked control changes
            If (v And EV.HVEN_CMD) Then RaiseEvent UsbHvEnableChange(Me.Status.HVEnabled)
            If (v And EV.SETPT_CMD) Then RaiseEvent UsbSetpointChange()
            ' Other
            If (v And EV.LOG_CHANGE) Then RaiseEvent StatusLogChange()
        End Sub

        ' Live status
        Public Status As New clsStatus(Me)
        Public Class clsStatus
            Private dev As Device
            Public Sub New(ByRef dev As Device)
                Me.dev = dev
            End Sub
            ' Faults -----------
            Public ReadOnly Property Undervoltage() As Boolean
                Get
                    Return (dev.MyStatus.StatusSys.LiveStatus And STAT_FLAGS.UVLO) <> 0
                End Get
            End Property
            Public ReadOnly Property Failed() As Boolean
                Get
                    Return (dev.MyStatus.StatusSys.LiveStatus And STAT_FLAGS.FAIL) <> 0
                End Get
            End Property
            Public ReadOnly Property Overtemp() As Boolean
                Get
                    Return (dev.MyStatus.StatusSys.LiveStatus And STAT_FLAGS.OVERTEMP) <> 0
                End Get
            End Property
            Public ReadOnly Property ComTimeout() As Boolean
                Get
                    Return (dev.MyStatus.StatusSys.LiveStatus And STAT_FLAGS.COM_TOUT) <> 0
                End Get
            End Property
            ' Permissives ------------
            Public ReadOnly Property HVEnabled() As Boolean
                Get
                    Return (dev.MyStatus.StatusSys.LiveStatus And STAT_FLAGS.HVEN) <> 0
                End Get
            End Property
            Public ReadOnly Property RemoteControl() As Boolean
                Get
                    Return (dev.MyStatus.StatusSys.LiveStatus And STAT_FLAGS.REMOTE) <> 0
                End Get
            End Property
        End Class

        ' Live status
        Public SubsystemFailed As New clsSubsystemFailed(Me)
        Public Class clsSubsystemFailed
            Private dev As Device
            Public Sub New(ByRef dev As Device)
                Me.dev = dev
            End Sub
            ' Fail flags -----------
            Public ReadOnly Property RelayDriver() As Boolean
                Get
                    Return (dev.MyStatus.StatusSys.FailFlags And FAIL_FLAGS.RELAY_DRV) <> 0
                End Get
            End Property
            Public ReadOnly Property RelayFeedback() As Boolean
                Get
                    Return (dev.MyStatus.StatusSys.FailFlags And FAIL_FLAGS.RELAY_FB) <> 0
                End Get
            End Property
            Public ReadOnly Property PosSupply() As Boolean
                Get
                    Return (dev.MyStatus.StatusSys.FailFlags And FAIL_FLAGS.POS_SUPPLY) <> 0
                End Get
            End Property
            Public ReadOnly Property NegSupply() As Boolean
                Get
                    Return (dev.MyStatus.StatusSys.FailFlags And FAIL_FLAGS.NEG_SUPPLY) <> 0
                End Get
            End Property
        End Class

        ' Status log
        Public StickyStatus As New clsStickyStatus(Me)
        Public Class clsStickyStatus
            Private dev As Device
            Public Sub New(ByRef dev As Device)
                Me.dev = dev
            End Sub
            Public Property Undervoltage() As Boolean
                Get
                    Return (dev.MyStatus.StatusSys.StatusLog And EVENT_LOG.UVLO) <> 0
                End Get
                Set(ByVal value As Boolean)
                    If Not value Then dev.ErrorDetect(BP2595_ClearEventLog(dev.MyDevAddr, EVENT_LOG.UVLO))
                End Set
            End Property
            Public Property SybsystemFailed() As Boolean
                Get
                    Return (dev.MyStatus.StatusSys.StatusLog And EVENT_LOG.FAIL) <> 0
                End Get
                Set(ByVal value As Boolean)
                    If Not value Then dev.ErrorDetect(BP2595_ClearEventLog(dev.MyDevAddr, EVENT_LOG.FAIL))
                End Set
            End Property
            Public Property Overtemp() As Boolean
                Get
                    Return (dev.MyStatus.StatusSys.StatusLog And EVENT_LOG.OVERTEMP) <> 0
                End Get
                Set(ByVal value As Boolean)
                    If Not value Then dev.ErrorDetect(BP2595_ClearEventLog(dev.MyDevAddr, EVENT_LOG.OVERTEMP))
                End Set
            End Property
            Public Property ComTimeout() As Boolean
                Get
                    Return (dev.MyStatus.StatusSys.StatusLog And EVENT_LOG.COM_TOUT) <> 0
                End Get
                Set(ByVal value As Boolean)
                    If Not value Then dev.ErrorDetect(BP2595_ClearEventLog(dev.MyDevAddr, EVENT_LOG.COM_TOUT))
                End Set
            End Property
            Public Property WasReset() As Boolean
                Get
                    Return (dev.MyStatus.StatusSys.StatusLog And EVENT_LOG.RESET) <> 0
                End Get
                Set(ByVal value As Boolean)
                    If Not value Then dev.ErrorDetect(BP2595_ClearEventLog(dev.MyDevAddr, EVENT_LOG.RESET))
                End Set
            End Property
            Public ReadOnly Property ValidPersonality() As Boolean
                Get
                    Return (dev.MyStatus.StatusSys.StatusLog And EVENT_LOG.ATTRDEF) <> 0
                End Get
            End Property
        End Class

        '**********************************************************************************************
        '***************************************  METERS  *********************************************
        '**********************************************************************************************

        ' Meter events
        Public Event MetersUpdate()     ' received meters update

        ' Meter monitor thread
        Private Delegate Sub GUIInvokerMetersUpdate(ByVal meters As BP2595_METERS)
        Private Sub MetersMonitorThreadFunction()
            Dim meters As New BP2595_METERS ' create meter data buffer
            ' Repeat until error or device closed:
            While (BP2595_GetMeterUpdate(MyDevAddr, meters, bp2595.INFINITE_WAIT) = BP2595_ERR.OK)         ' Wait for meter update from device.
                MyGuiContext.BeginInvoke(New GUIInvokerMetersUpdate(AddressOf RaiseMeterUpdate), meters)    ' Schedule call to RaiseMeterUpdate in GUI context.
            End While
        End Sub
        Private Sub RaiseMeterUpdate(ByVal meters As BP2595_METERS)    ' Forward received meter update to application
            MyMeters = meters           ' Sync local meters image to device
            RaiseEvent MetersUpdate()   ' Notify app that meters have been updated
        End Sub

        ' Meter properties
        Public ReadOnly Property Meters() As BP2595_METERS
            Get
                Return MyMeters  ' engr units
            End Get
        End Property
        Public Property MeterInterval() As UInt16
            Get
                Dim interval As UInt16
                ErrorDetect(BP2595_GetMeterInterval(MyDevAddr, interval))
                Return interval
            End Get
            Set(ByVal value As UInt16)
                ErrorDetect(BP2595_SetMeterInterval(MyDevAddr, value))
            End Set
        End Property

        '**********************************************************************************************
        '***********************************  DEVICE CONTROL  *****************************************
        '**********************************************************************************************

        ' Setpoint
        Public Setpoint As New clsSetpoint(Me)
        Public Class clsSetpoint
            Private dev As Device
            Public Sub New(ByRef dev As Device)
                Me.dev = dev
            End Sub
            Public Property Volts() As Int32
                Get
                    Return dev.MyStatus.SetpointV
                End Get
                Set(ByVal volts As Int32)
                    dev.ErrorDetect(BP2595_ProgramSetpoint(dev.MyDevAddr, volts))
                End Set
            End Property
        End Class

        ' HV enable
        Public Property HvEnable() As Boolean
            Get
                Return (MyStatus.StatusSys.LiveStatus And STAT_FLAGS.HVEN) <> 0
            End Get
            Set(ByVal enable As Boolean)
                ErrorDetect(BP2595_ProgramHvEnable(MyDevAddr, enable))
            End Set
        End Property

        '**********************************************************************************************
        '***********************************  DEVICE PERSONALITY  *************************************
        '**********************************************************************************************

        Public Function GetAttributes() As Device.BP2595_ATTR  ' Device personality
            Dim attr As New Device.BP2595_ATTR
            ErrorDetect(BP2595_GetPersonality(MyDevAddr, attr))
            Return attr
        End Function

        '**********************************************************************************************
        '******************************************  OTHER  *******************************************
        '**********************************************************************************************

        Public Property ComWatchdogInterval() As UInt16
            Get
                Dim interval As UInt16
                ErrorDetect(BP2595_GetComWatchdog(MyDevAddr, interval))
                Return interval
            End Get
            Set(ByVal value As UInt16)
                ErrorDetect(BP2595_SetComWatchdog(MyDevAddr, value))
            End Set
        End Property
        Public ReadOnly Property FirmwareVersion() As bp2595.VERSION    ' Get device firmware version
            Get
                Return MyVersionInfo.DevFw
            End Get
        End Property
        Public ReadOnly Property HardwareRev() As Char    ' Get device hardware version
            Get
                Return MyVersionInfo.PwbRev
            End Get
        End Property

    End Class ' Device

End Module ' c2595


